%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONNECTIVITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize the code
clear variables; close all; clc; tic
close all; clc; tic
disp(datestr(now));

%% INITIALIZE THE CODE

if ispc == 1
    slash = '\';
elseif isunix == 1
    slash = '/';
end
directory = 'input';

% load the input DATA
nomefile = strcat(directory,slash,'data');
load(nomefile,'data');

% load temp_1.mat
% data.node_data = temp_1;

% set the domain
nnode = length(data.node_data);
[nrows,ncols]=size(data.FLA);
pxl_dim = 36; %[m]

% find the k-index of the outlet
outlet = find(data.FLA==max(max(data.FLA)));

% find the node number of the outlet
aux = cat(1,data.node_data.k);
node_out = find(aux == outlet);

% set input parameters for the connectivity model
h_star = 250; %[mm] da scegliere
h0 = 0.04; %[d^delta/mm^3delta-1] fissato
delta = 0.3; % fissato

% initialize the normalization constant C
NC_q = zeros(nnode,1);
NC_h = zeros(nnode,1);
cdfQ_check = zeros(nnode,1);
cdfH_check = zeros(nnode,1);
atom = zeros(nnode,1);
var = zeros(nnode,1);
CV = zeros(nnode,1);
kur = zeros(nnode,1);
% % initialize threshold parameters
% qstar = zeros(nnode,1);
% hstar = zeros(nnode,1);

% extract the field about node index (K-field) from both data structures
% (nodes and links)
Km = cat(1, data.link_data.Km);
k = cat(1, data.node_data.k);

q_media = zeros(nnode,1);

% update the node_data structure
% loop over the nodes
for jn = 1:nnode
    
% calculate the normalization constant C
    [NC_q(jn), cdfQ_check(jn), atom(jn)] =...
        find_normconst (data.node_data(jn).a, data.node_data(jn).alphaP, data.node_data(jn).kappa,...
        data.node_data(jn).lambda);

% calculate the cdf_Q   
    [data.node_data(jn).pdfQ_cont, data.node_data(jn).cdf, q_media(jn)] = ...
        pdf_Q (NC_q(jn), data.node_data(jn).a, data.node_data(jn).alphaP, data.node_data(jn).kappa,...
        data.node_data(jn).lambda, data.node_data(jn).q, jn);    

% -------------------------------------------------------------------------
% [nc_q(jn,1), cdf(jn,1)] = analytical_pdf (data.node_data(jn).a, data.node_data(jn).alphaP, data.node_data(jn).kappa,...
%         data.node_data(jn).lambda, data.node_data(jn).q);
% -------------------------------------------------------------------------
    
% calculate the threshold discharge for each node [mm/d]
    [data.node_data(jn).qstar] = ((h_star./h0).^(1./delta))./(data.node_data(jn).fla*(pxl_dim^2).*10^6);

% calculate the connectivity index for each node
    [NC_h(jn), cdfH_check(jn), data.node_data(jn).pdfH, data.node_data(jn).frag, data.node_data(jn).h_media,data.node_data(jn).meanHS, var(jn), kur(jn)] = ...
            pdf_H_BIS (NC_q(jn),data.node_data(jn).a, data.node_data(jn).alphaP, data.node_data(jn).kappa,...
            data.node_data(jn).lambda,h0 ,delta,data.node_data(jn).fla, pxl_dim, h_star, jn);
        
        CV(jn) = sqrt(var(jn))/data.node_data(jn).h_media;
        
% round the fragmentation index to the second decimal
    if data.node_data(jn).frag < 0
        data.node_data(jn).frag = 0;
    end
%     data.node_data(jn).frag = 1/100*floor(100*data.node_data(jn).frag);
%     data.node_data(jn).frag = round(data.node_data(jn).frag,2);
    
% round the connectivity index to the second decimal
    data.node_data(jn).cnt = 1-data.node_data(jn).frag; 
    
end 

% set the normalized connectivity
cnt = [data.node_data.cnt]';
norm_cnt = cnt./max(cnt);

%  set normalized connectivity into data structure
for jn = 1:nnode
    data.node_data(jn).norm_cnt = round(norm_cnt(jn,1),2);
end 

%% FRAGMENTATION FORCING

% SET A SPECIFIC NODE FRAGMENTATION
if (0)
data.node_data(9).frag = 1;
end

% SHUFFLE FRAGMENTATION FIELD
if (0)
frg = cat(1,data.node_data.frag);

% randomize the order
new_frg = frg(randperm(length(frg)));

% insert the new vector in the structure
    for js = 1:length(frg)
        data.node_data(js).frag = new_frg(js,1);
    end
end

% LOAD A NEW FRAGMENTATION CONFIGURATION
if (0)
new_frg = load('fragmentation.txt');

% insert the new vector in the structure
    for js = 1:length(new_frg)
        data.node_data(js).frag = new_frg(js,1);
    end
end
% -------------------------------------------------------------------------
%%
% update the link_data structure, adding the information of each UPSTREAM NODE to
% the corresponding link
for jn=1:nnode
    data.link_data(jn).a = data.node_data(Km(jn)==k).a;
    data.link_data(jn).alphaP = data.node_data(Km(jn)==k).alphaP;
    data.link_data(jn).lambda = data.node_data(Km(jn)==k).lambda;
    data.link_data(jn).kappa = data.node_data(Km(jn)==k).kappa;
    data.link_data(jn).frag = data.node_data(Km(jn)==k).frag;
    data.link_data(jn).cnt = data.node_data(Km(jn)==k).cnt;
    data.link_data(jn).norm_cnt = data.node_data(Km(jn)==k).norm_cnt;
end

%%
data.link_data(end) = [];

% calculate the adjacency matrix
adj.dist = adj_matrix(data.link_data, 'IDm', 'IDv', 'L');
adj.alphaP = adj_matrix(data.link_data, 'IDm', 'IDv', 'alphaP');
adj.lambda = adj_matrix(data.link_data, 'IDm', 'IDv', 'lambda');
adj.kappa = adj_matrix(data.link_data, 'IDm', 'IDv', 'kappa');
adj.a = adj_matrix(data.link_data, 'IDm', 'IDv', 'a');
adj.frag = adj_matrix(data.link_data, 'IDm', 'IDv', 'frag');
adj.cnt = adj_matrix(data.link_data, 'IDm', 'IDv', 'cnt');
adj.norm_cnt = adj_matrix(data.link_data, 'IDm', 'IDv', 'norm_cnt');

%% CONNECTIVITY METRICS

% calculate the path of each node to all other nodes (es. 1-4: npath=[1 2 3 4])
[Lpath, npath] = find_npath (nnode, adj.dist);

% calculate the connectivity measures
[CM.Clocal, CM.Cpath, CM.Cnode, CM.Cntw] = ...
    connectivity (nnode,data.node_data, adj.cnt, adj.dist, npath, Lpath, 1);

% nlink = nnode-1;
% 
% [CM2.Clocal, CM2.Cpath, CM2.Cnode, CM2.Cntw] = ...
%     connectivity2 (nlink,data.link_data, adj.cnt, adj.dist, npath, Lpath, 1);

% set normalized Clocal
Clocal = CM.Clocal;
Cnorm = zeros(nnode,1);
for jn = 1:nnode
    Cnorm (jn) = Clocal(jn)/max(Clocal);
end
% set raster of Clocal
C_raster = array2raster(Clocal, nrows, ncols, data.link_ntw);

% set raster of normalized Clocal
Cnorm_raster = array2raster(Cnorm, nrows, ncols, data.link_ntw);

q =[data.node_data.q]';

% set raster of streamflows
q_raster = array2raster(q, nrows, ncols, data.link_ntw);


% set normalized HS
HS=cat(1,data.node_data.meanHS);
HS_norm = HS/max(HS);
% set raster of  HS
HS_raster = array2raster(HS, nrows, ncols, data.link_ntw);

% set raster of normalized HS
[data(:).HSraster] = array2raster(HS_norm, nrows, ncols, data.link_ntw);

% select the Path Connectivity of the outlet
Cpath_out = CM.Cpath(node_out,:)';
% set normalized Cpath_outlet
Cpath_out_norm = Cpath_out/max(Cpath_out);
% set raster of normalized Cpath_outlet
Cpath_out_raster = array2raster(Cpath_out_norm, nrows, ncols, data.link_ntw);
% set raster of Cpath_outlet
Cout_raster = array2raster(Cpath_out, nrows, ncols, data.link_ntw);


% Outlet connectivity: connectivity of the outlet in respect of all other nodes 
Cout = sum(Cpath_out,1)./(nnode-1);
Cout_norm = sum(Cpath_out_norm,1)./(nnode-1);

% EFFECTIVE Outlet connectivity: connectivity of the outlet in respect of all other nodes considering the HS 
Cout_eff = sum(Cpath_out.*HS)/sum(HS,1);

Cout_eff_norm = sum(Cpath_out_norm.*HS_norm)/sum(HS_norm,1);


% save connectivity measures
[data(:).CM] = CM;

fprintf('Cnetwork         :%8.4f \n',CM.Cntw);
fprintf('Cout             :%8.4f \n',Cout);
fprintf('Cout_eff         :%8.4f \n',Cout_eff);
fprintf('SI               :%8.4f \n',Cout_eff/Cout);
fprintf('Cout_norm        :%8.4f \n',Cout_norm);
fprintf('Cout_eff_norm    :%8.4f \n',Cout_eff_norm);
fprintf('SI_norm          :%8.4f \n',Cout_eff_norm/Cout_norm);

% % save data structure
% save(nomefile,'data');


%%
% PLOTS

% % plot raster of streamflows
% plotting(q_raster);
% title('Mean discharge','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'q [mm/d]');
% 
% 
% % plot raster of Clocal
plotting(C_raster);
title('Clocal','FontSize', 25); % 'fontname', 'times new roman');
title(colorbar,'C_{local} [-]');


% % plot raster of Clocal normalized
% plotting(Cnorm_raster);
% title('Cnorm','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'C_{norm} [-]');


% % plot raster of Path Connectivity of the outlet
% plotting(Cout_raster);
% title('Paths','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'Path [-]');
% 
% % plot raster of Path Connectivity of the outlet
% plotting(Cpath_out_raster);
% title('Normalized paths','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'Path_{norm} [-]');
% 
% % plot raster of HS
% plotting(HS_raster);
% title('Habitat suitability','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'HS [-]');
% 
% % plot raster of HS
% plotting(data.HSraster);
% title('Normalized suitability','FontSize', 25); % 'fontname', 'times new roman');
% title(colorbar,'HS_{norm} [-]');

%%
% Create shapefile
% shp = shapefile(data.HSraster, pxl_dim);
% shapewrite(shp,'shp_HSdry.shp');
% 
% % shp = shapefile(Cpath_out_raster, pxl_dim);
% % shapewrite(shp,'shp_path.shp');

shp = shapefile(C_raster, pxl_dim);
shapewrite(shp,'shp_250_1.shp');

% shp = shapefile(q_raster, pxl_dim);
% shapewrite(shp,'shp_q.shp');

%%


% end of the code
disp(datestr(now));

toc

