%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ECO_HYDROLOGICAL CONNECTIVITY
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% INITIALIZE THE CODE
if ispc == 1
    slash = '\';
elseif isunix == 1
    slash = '/';
end
directory = 'input';

% load the input DATA
nomefile = strcat(directory,slash,'data');
load(nomefile,'data');

% set the domain
nnode = length(data.node_data);
[nrows,ncols]=size(data.FLA);
pxl_dim = 36; %[m]

% find basin outlet
k_out = find(data.FLA == max(max(data.FLA)));
n_out = find(cat(1,data.node_data.fla) == max(max(data.FLA)));

% find the source of the domain (FLA=1)
source_percorsi=find(data.FLA==1);
[n_percorsi,~]=size(source_percorsi);

% find for each pixel of the domain the next downstream
[vettore_scarico, dist, notnan] = find_next_downstream(data.FLD);

% find the path from each source (FLA=1) to the outlet
[percorsi] = paths_from_source(n_percorsi, source_percorsi, vettore_scarico, k_out); 

% build the raster of the nodes
[raster_nodes, nnodes, node_net] = nodes2raster (data.rete, data.node_data);

%save raster node information
[data(:).raster_nodes] = raster_nodes;
[data(:).node_ntw] = node_net;

% find the path from each node to the outlet
[path_out, data.node_data] = path2out (nnodes, data.node_data, raster_nodes, k_out, vettore_scarico, dist);

% save node path information
[data(:).path_out] = path_out; 

% find the habitat suitability for each node
[data.node_data] = find_HS(data.node_data, nnodes,2,1);

% extract the array from the structure
HS = cat(1,data.node_data.HS);

% set connectivity rasters
[data(:).HSraster] = array2raster(HS, nrows, ncols, data.link_ntw);

% plot raster of HS
plotting(data.HSraster');

% select the Path Connectivity of the outlet
Cpath_out = data.CM.Cpath(n_out,:)';

% Cpath_out = Cpath_out./max(Cpath_out);

% set the matrix from path_out array
Cpath_out_raster = array2raster(Cpath_out, nrows, ncols, data.link_ntw);

% plot the Path Connectivity of the outlet
plotting(Cpath_out_raster');

% % set the Node Connectivity of the outlet
% Cnode_out = sum(Cpath_out)/(nnode-1); 
% 
% % set Path Connectivity of the outlet considering the habitat suitability
% ecopath_out = Cpath_out.*HS;
% 
% % set the matrix from ecopath_out array
% ecopath_raster = array2raster(ecopath_out, nrows, ncols, data.link_ntw);
% 
% % plot Path Connectivity of the outlet considering the habitat suitability
% plotting(ecopath_raster');
% 
% % find the eco Node Connectivity of the outlet
% Ceconode_out = sum(ecopath_out)/(nnode-1); 

% % save data structure
% save(nomefile,'data');