%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% RAINFALL SCENARIOS
% 
% Authors: Silvia Garbin
% Creation: march 2016
% Last update:march 2017
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize the code
clear variables; close all; clc; tic
disp(datestr(now));

%% INITIALIZE THE CODE

% set the delay of printing in command window
jprint = 10;

% set operative system
if ispc == 1
    slash = '\';
elseif isunix == 1
    slash = '/';
end

% set input directory
directory = 'input';

% load the input DATA
nomefile = strcat(directory,slash,'data');
load(nomefile,'data');

% set the domain
[nrows,ncols]=size(data.FLA);

% find the outlet information
outlet = find(data.FLA==max(max(data.FLA)));
outlet_fla = max(max(data.FLA));
outlet_basin = data.node_data(cat(1,data.node_data.k)==outlet).m;

% set frequencies of the process:
% time frequency
lambdaT = 1.5; 

% space frequency
lambdaX = 0.5/ncols;
lambdaY = 0.5/nrows;

% find the frequency of the process
mean_measure = lambdaT*lambdaX*lambdaY;
lambda = mean_measure*ncols*nrows;

% set rainfall cells information
% mean cell radius
r_m = 15;

% mean alpha
alpha_m = 2000;% [5-30 mm]

% set the number of days simulated
days =3000;

% find the counting variable: the number of events per day
Nt = poissrnd(lambda,1,days)';

% initialize the arrays
nnode = length(data.node_data);
pcum = zeros(nnode,1);
rate = zeros(nnode,1);

% initialize the array
dcr = zeros(nnode,1);
% initialize the array
dr = zeros(nnode,1);

% initialize the matrix of cumulated rainfall
cumrain = zeros(nrows, ncols);

% initialize the matrix of cumulated rate
cumrday = zeros(nrows, ncols);

% initialize the cumulative rainfall over the basin area
rain_basin = 0;

% initialize the cumulative rainyday associated to the basin area
rday_basin = 0;

% if you assign 1 you save, with 0 you don't save
sw_rain = 0;
if sw_rain
    rain_storage = repmat(struct('rain',0,'rain_cutted',0,'cumrain',0),days,0);
end

% initialize time
ti = cputime;

% % Create figure
% figure1 = figure('units','normalized','outerposition',[0.03 0.21 0.94 0.75]);
% maxnow = 0;

% % initialize structure
% timeplot = 365; 
% saved_rain = repmat(struct('space',0,'j23',0,'j17',0),timeplot,1);

% % initialize the counter
% tt = 1;

% loop over days
for t = 1:days
    
% initialize the matrix
%     centers = zeros(Nt(t),4);
%%    
% find coordinates setting the type of centre's distribution (flagR and flagC).
% (1) UNIFORM distribution
% (2) linear INCREASING distribution
% (3) linear DECREASING distribution
% [event] = find_coord (flagR, flagC, nrow, ncol, nevents);
  
    event = find_coord (1, 1, nrows, ncols, Nt(t), t);
%%
% find the cells radius   
    radius = find_radius (Nt(t), r_m);
%%    
% find the ALPHA pattern
% (3 input) UNIFORM pattern
% (7 input) LINEAR pattern along X or Y
% (8 input) LINEAR pattern along X and Y

% [alpha] = ...
% find_alpha(mean_alpha, nevent, t, event, ncol, nrow, flagTYPE1, flagTYPE2) 

% find the type pattern (flagTYPE)
% (1-101) Strong increase
% (2-102) Strong decrease
% (3-103) Weak increase
% (4-104) Weak decrease

% x-direction goes from flagTYPE=1 to flagTYPE=4
% y-direction goes from flagTYPE=101 to flagTYPE=104

%     alpha = find_alpha (alpha_m, Nt(t), t, event, ncols, nrows, 1, 101);
    alpha = find_alpha (alpha_m, Nt(t), t);


    intensity = find_intensity(Nt(t), alpha);
%%    
% build the centre matrix
    centers = [event, radius, intensity];
%%
% build the rain raster
%     rain = rain_raster(centers, nt, nrows, ncols);
    rain = rain_raster(centers, Nt(t), nrows, ncols);
    
% if sw_rain = 1,
    if sw_rain
        rain_storage(t).rain = rain;
    end
    
% % cut 1mm of rainfall
%     rain = significant_threshold(rain); 
    
    if sw_rain
        rain_storage(t).rain_cutted = rain;
    end
    
% calculate the daily cumulated rainfall (dcr) and the daily rate (dr) of each node    
    for jn = 1:nnode
        [dcr(jn), dr(jn)] = ...
            set_rain_node(rain, data.node_data(jn).m, data.node_data(jn).fla);          
    end

    
% calculate the cumulative rainfall 
    cumrain = cumrain + rain;
   
% calculate the cumulative rainfall over the basin area
    [cr_b, r_b] = set_rain_node(rain, outlet_basin, outlet_fla);
    rain_basin = rain_basin + cr_b;
    
% calculate the cumulative rate
    rainyday = rain;
    rainyday(rainyday>0)=1;
    cumrday = cumrday + rainyday;
    
% calculate the cumulative rainydays associates to the basin area
    rday_basin = rday_basin + r_b;
    
% % save the intensity of the basin #23 and #17    
%         if t>2130  && t<2496
%             jn = 23;
%             saved_rain(tt).j23 = dcr(jn);
%             jn = 17;
%             saved_rain(tt).j17 = dcr(jn);
%             
% % save current rain distribution
%             saved_rain(tt).space = rain;
%             
% % update the counter
%             tt = tt + 1;
%         end
%%
%         subplot1 = subplot(1,2,1,'Parent',figure1);
%         imagesc(rain)
%         axis('square')
%         set(gca,'xtick',[],'ytick',[]);
%         title('rain [mm/d]', 'fontname','times new roman','fontsize',15)
% % Set the remaining axes properties
%         maxnow = max([maxnow max(max(rain))]);
%         set(subplot1,'CLim',[0 maxnow],'Layer','top');
% % Create colorbar
%         colorbar('peer',subplot1);
%         
%         subplot2 = subplot(1,2,2,'Parent',figure1);
%         imagesc(cumrain)
%         axis('square')
%         set(gca,'xtick',[],'ytick',[]);
%         title(['cumulated rain [mm] in ' num2str(t) ' days'], 'fontname','times new roman','fontsize',15);
%         
% % Set the remaining axes properties
%         set(subplot2,'CLim',[0 3000],'Layer','top');
% % Create colorbar
%         colorbar('peer',subplot2);
%         pause(0.1) 
%%
    if sw_rain
        rain_storage(t).cumrain = cumrain;
    end

% calculate the cumulated rainfall of each node
    for jn = 1:nnode
        pcum(jn) = pcum(jn) + dcr(jn);  
% calculate the rate of each node
        rate(jn) = rate(jn) + dr(jn); 
    end

% print status
    if mod(t,jprint)==0
        te = cputime;
        fprintf('day %u of %u (%6.2f %%): %8.2f s (%8.4f s) \n', ...
            t, days, t/days*100,te-ti,(te-ti)/jprint);
        ti = cputime; 
        
        fprintf('  daily mean rain %6.2f mm/d \n', rain_basin/t);
        fprintf('  daily mean rate %6.2f 1/d \n', rday_basin/t);         
    elseif  t == days
        te = cputime;
        fprintf('day %u of %u (%6.2f %%): %8.2f s (%8.4f s) \n', ...
            t, days, t/days*100,te-ti,(te-ti)/mod(t,jprint));   
        
        fprintf('  daily mean rain %6.2f mm/d \n', rain_basin/t);
        fprintf('  daily mean rate %6.2f 1/d \n', rday_basin/t); 
    end 
    
%     if mod(t,jprint)==0
%         subplot(1,2,1,'Parent',figure1)
%         hold on
%         plot(t, (rain_basin/t),'-bo');
%         xlabel('time');
%         ylabel('mm/d');
%         pause(0.1)
%         subplot(1,2,2,'Parent',figure1)
%         hold on
%         plot(t, rday_basin/t,'-r*');
%         xlabel('time');
%         ylabel('1/d');
%         pause(0.1)
%     end
   
% end of the loop over time
end

% save cumulated rainfall
[data(:).basin.cumrain] = rain_basin;
[data(:).basin.meanrain] = rain_basin/days;
[data(:).basin.lambdaP] = rday_basin/days;
[data(:).basin.alphaP] = data(:).basin.meanrain/data(:).basin.lambdaP;

% calculate lambdaP and alphaP for each node
for jn = 1:nnode
    [data.node_data(jn).meanrain] = pcum(jn)/days;
    [data.node_data(jn).lambdaP] = rate(jn)/days;
    [data.node_data(jn).alphaP] = ...
        [data.node_data(jn).meanrain] / [data.node_data(jn).lambdaP];
end

% save data structure
save(nomefile,'data');

% end of the code
disp(datestr(now));
toc

% % save rainfall
% save('rainfall','saved_rain');