%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUBBASINS EXTRACTION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize the code
clear variables; close all; clc; tic;
disp(datestr(now));

%% INITIALIZE THE CODE

% check operative system
if ispc == 1
    slash = '\';
elseif isunix == 1
    slash = '/';
end
directory = 'input';

% load the input FLD and FLA
if (1)
    nomefile = strcat(directory,slash,'data_input',slash,'flowdir.txt');
    FLD = load(nomefile);
    nomefile = strcat(directory,slash,'data_input',slash,'flowacc.txt');
    FLA = load(nomefile);
    data = struct('FLD',FLD,'FLA',FLA);
    clear flowdir flowacc FLD FLA
end


% find basin outlet
k_out=  find(data.FLA == max(max(data.FLA)));

[i_out,j_out] = ind2sub(size(data.FLA)',k_out);

[nrows, ncols] = size(data.FLA);
pxl_dim = 36; %[m]
    
% define threshold on contributing area (number of pixel)
area_tresh = 10; 

% calculate network and parameter a
[net, a_geom, ~] = ...
    a_geomorf(data.FLD,data.FLA, nrows, ncols, pxl_dim, area_tresh);

[data(:).rete]=net;
clear net

% calculate drainage density
DD = ((nansum(nansum(data.rete))*pxl_dim + nansum(nansum(data.rete))*...
    pxl_dim*sqrt(2))/2)/max(max(data.FLA));
% fprintf ('DD         : %10.3f 1/km \n',DD);

% find flow accumulation of channel pixels
FLA_net = data.rete.*data.FLA;
[data(:).FLA_rete] = FLA_net;
clear FLA_net

% find the threshold for the new network
net_tresh = min(data.FLA_rete( data.FLA_rete > 0));

%%
% find the source of the domain (FLA=1)
source_percorsi=find(data.FLA==1);
[n_percorsi,~]=size(source_percorsi);

% find for each pixel of the domain the next downstream
[vettore_scarico, dist, notnan] = find_next_downstream(data.FLD);

% find the path from each source (FLA=1) to the outlet
[percorsi] = paths_from_source(n_percorsi, source_percorsi, vettore_scarico, k_out); 
%%
    
% find junctions
[~, junction] = ...
    find_junction (data.FLD, data.FLA, nrows, ncols, net_tresh);

% find channel heads
heads = find_head (data.rete, nrows, ncols, vettore_scarico);

% add outlet in the heads array 
jn = length(heads)+1;
heads(jn).i = i_out;
heads(jn).j = j_out;
heads(jn).k = k_out;

nheads = length(heads);

% build nodes as merging of junctions and channel heads
nodes = [junction; heads];

% sort nodes with respect to index k
for jk = 1:length(nodes)-1
    for jk1 = jk+1:length(nodes)
        if nodes(jk1).k < nodes(jk).k
            aux = nodes(jk);
            nodes(jk) = nodes(jk1);
            nodes(jk1) = aux;
        end
    end
end

% check for potential node repetitions and remove them
jk = 1;
while jk <= length(nodes)-1
	if nodes(jk+1).k == nodes(jk).k
        nodes(jk+1) = [];
        jk = 1;
	else
        jk = jk + 1;
	end
end

% build the raster of the nodes
[raster_nodes, nnodes, node_net] = nodes2raster (data.rete, nodes);

[data(:).node_ntw] = node_net;

% add ID to each node
for jn = 1:nnodes
    nodes(jn).ID = jn;
end

%%
% find the path from each head to the outlet
[path_heads, max_knode] = path4heads (nheads, heads, raster_nodes, k_out, vettore_scarico);

% save node path information
[data(:).path_node] = path_heads;
   
%%
[link_data] =  find_link (nheads, heads, nnodes, raster_nodes, nrows, ncols, k_out, vettore_scarico, dist, nodes);
% plot_link_data(link_data,nrows,ncols);

% save links information
[data(:).link_data] = link_data;

% create raster of links
[link_ntw] = link2raster(nrows, ncols, nnodes, vettore_scarico, data.link_data);

[data(:).link_ntw] = link_ntw;
%%
[node_data] = ...
    subbasins_nodes(nrows, ncols, n_percorsi, percorsi, raster_nodes, nnodes); 

[data(:).node_data] = node_data;
%%
% extract k-field from both data structures
Km = cat(1, data.link_data.Km);
k = cat(1, data.node_data.k);

% update the node_data structure
% loop over the nodes
for jn = 1:nnodes
    
% add the downstream node
    data.node_data(jn).nodeV = data.link_data(k(jn)==Km).Kv;
    
% add the downstream link
    data.node_data(jn).linkL  = data.link_data(k(jn)==Km).L; 
    
% add the corresponding a_geomorf parameter    
    data.node_data(jn).a = a_geom(data.node_data(jn).k);
    
% % set the condition to avoid a=1
%     if data.node_data(jn).a == 1
%         data.node_data(jn).a = data.node_data(jn).a + 0.1;
%     end
    
end

% save data structure
nomefile = strcat(directory,slash,'data');
save(nomefile,'data');

% save('matrix_subbasin_channel.mat','matrix_subbasin_channel','-v7.3');

 %% PLOTS
% 
% % plot a_geomorf
% figure()
% [silvia] = create_my_colormap(100, [255 255 255]/255, [0 49 83]/255, [200 200 200]/255);
% imagesc(a_geom);
% colormap(silvia)
% caxis([0, max(a_geom(:))])
% colorbar;
% set(gca,'XTick',[],'YTick',[],'FontSize', 10, 'fontname', 'times new roman');
% title('a [-]','FontSize', 30, 'fontname', 'times new roman');
% axis square
% 
% % plot rete
% figure()
% [ntw] = create_my_colormap(2, [255 255 255]/255, [0 0 255]/255);
% imagesc(net);
% colormap(ntw)
% caxis([0, max(net(:))])
% set(gca,'XTick',[],'YTick',[],'FontSize', 10, 'fontname', 'times new roman');
% title('Network','FontSize', 30, 'fontname', 'times new roman');
% axis square

toc