%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PORPORATO WATER BALANCE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Initialize the code
clear variables; close all; clc; tic
disp(datestr(now));

%% INITIALIZE THE CODE

if ispc == 1
    slash = '\';
elseif isunix == 1
    slash = '/';
end
directory = 'input';

% load the input DATA
nomefile = strcat(directory,slash,'data');
load(nomefile,'data');

% set the domain
[nrows,ncols]=size(data.FLA);
pxl_dim = 36; %[m]

% find the k-index of the outlet
outlet = find(data.FLA==max(max(data.FLA)));

% find the node number of the outlet
aux = cat(1,data.node_data.k);
node_out = find(aux == outlet);

% set the domain
nnode = length(data.node_data);

% set input parameters for Porporato Water Balance
% field capacity:
s1 = 0.5;
% wilting point:
sw = 0.2;
% porosity:
n = 0.35;
% root zone [mm]  [winter=1500mm, fall=800mm, summer=470mm, spring=135mm]
Zr = 130;

alpha1 = 5; %[mm]
lambda1 = 0.5; %[1/d]

% set the ETC gradient [3(dry) - 0.5(wet) mm/d]
% inizialize the matrix
etc_raster = zeros(nrows,ncols);
etc_dd = zeros (nnode,1);
% etc_dd = daily evapotranspiration [mm/d]

% RANDOM PATTERN OF ETC (L.Bertassello)
% sigma: standard deviation , i.e. root-mean-square roughness Rq(m)
% H: Hurst exponent (roughness exponent), 0<= H <= 1
% It relates to the fractal dimension of a surface by D = 3-H.
% Lx: length of topography in x direction.
% m: number of pixels in x 
% n: number of pixels in y
% qr (((OPTIONAL parameter)))):

if (0)
    % set parameters:
    sigma = 2;
    H = 0.95;
    Lx = ncols*pxl_dim;
    m_cols = ncols;
    n_cols = nrows;
    qr = 0;

    etc_raster = artificial_surf(sigma, H, Lx, m_cols , n_cols, qr);
    min_etc = abs(min(min(etc_raster)));
    etc_raster = (etc_raster + min_etc)/2;
    etc_raster = flipud(etc_raster);
end
    
% UNIFORM PATTERN
if (1)
etc_raster(etc_raster==0) = 3.5;
end
% DECREASING/INCREASING N-S
if (0)
etc_N = 10;
etc_S = 0;
slope = (etc_S - etc_N)/(nrows-0);
q = etc_N;
for j = 1:nrows
    etc_raster(j,:) = slope*j + q;
end
end
% DECREASING/INCREASING W-E
if (0)
etc_W = 10;
etc_E = 0;
slope = (etc_E - etc_W)/(ncols-0);
q = etc_W;
    for j = 1:ncols
        etc_raster(:,j) = slope*j + q;
    end
end

% set etc value to the nodes
for jn = 1:nnode
    [etc_dd(jn,1),~] = set_rain_node(etc_raster,data.node_data(jn).m, data.node_data(jn).fla);
    [data.node_data(jn).etc] = etc_dd(jn,1);
end   

% calculate the soil water storage capacity [mm]
w0 = (s1-sw)*n*Zr;

% fprintf(' jn    w0    alphaP    lambdaP   Gamm    Eta    Par1    ga1    ga2   Ga   ratio \n');
% loop over the nodes
for jn = 1:nnode
    [data.node_data(jn).fi, data.node_data(jn).lambda] = ...
        find_lambdaPORPORATO(w0, etc_dd(jn,1), data.node_data(jn).lambdaP, data.node_data(jn).meanrain);
%     [data.node_data(jn).fi, data.node_data(jn).lambda] = ...
%         find_lambdaBUDYKO(etc_dd(jn), data.node_data(jn).alphaP, data.node_data(jn).lambdaP);
%     
    [data.node_data(jn).kappa] = ...
        find_kappa(data.node_data(jn).alphaP, data.node_data(jn).lambda, data.node_data(jn).a);
        
    [data.node_data(jn).q] = data.node_data(jn).lambda * data.node_data(jn).alphaP;
end
%%
% basin runoff coefficient
fi_basin = data.node_data(node_out).fi;
fprintf(' basin runoff coeff.:%8.4f \n',fi_basin);
etc_basin = data.node_data(node_out).etc;
fprintf(' basin etc          :%8.4f \n',etc_basin);

% alphaP = cat(1,data.node_data.alphaP);
% soil_capacity = w0/alphaP(node_out,1);
% fprintf(' soil capacity  :    %8.2f \n',soil_capacity);
% 
% meanrain = cat(1,data.node_data.meanrain);
% Di = etc_dd(node_out,1)/meanrain(node_out,1);
% fprintf(' aridity index  :    %8.2f \n',Di);


%%\


% save data structure
save(nomefile,'data');


% end of the code
disp(datestr(now));

toc




% Create shapefile
shp = shapefile(data.rete, pxl_dim);
shapewrite(shp,'rete.shp');
