%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% ALPHA_GEOMORPH
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [rete, ALPHA_G, Njunction] = ...
    a_geomorf(flowdir,flowacc, nrow, ncol, pixdim, threshold)

% initialize time
ti0 = cputime;

% define n_ flow directions

    MAX_DRAINING_PIXEL_IN_JUNCTION=8;

    N_DIR=64;
    NE_DIR=128;
    E_DIR=1;
    SE_DIR=2;
    S_DIR=4;
    SO_DIR=8;
    O_DIR=16;
    NO_DIR=32;

% initialize LU (lenght upstream) matrix 
    LU=zeros(nrow,ncol);

% Calculation of LU (lenght upstream) matrix using flow accumulation
% threshould. For equal contributing area, it takes the first one (correct)
% direction.
temp=0;
for i=0:nrow-1
    for j=0:ncol-1
        k=i;
        m=j;
        t=0;
        if flowacc(i+1,j+1)>threshold
            while flowacc(k+1,m+1)>threshold
            
            tm=m;
            tk=k;
            tt=t;
            
            if m>0
                if flowdir(k+1,m-1+1)==E_DIR
                    tk=k+0;
                    tm=m-1;
                    tt=t+1;
                    temp=flowacc(k+1,m-1+1);
                end
            end
            
            if k<nrow-1 && m>0
                if flowdir(k+1+1,m-1+1)==NE_DIR && flowacc(k+1+1,m-1+1)>temp
                    tk=k+1;
                    tm=m-1;
                    tt=t+sqrt(2);
                    temp=flowacc(k+1+1,m-1+1);
                end
            end
            
            if k<nrow-1
                if flowdir(k+1+1,m+1)==N_DIR && flowacc(k+1+1,m+1)>temp
                    tk=k+1;
                    tm=m+0;
                    tt=t+1;
                    temp=flowacc(k+1+1,m+1);
                end
            end
            
            if k<nrow-1 && m<ncol-1
                if flowdir(k+1+1,m+1+1)==NO_DIR && flowacc(k+1+1,m+1+1)>temp
                    tk=k+1;
                    tm=m+1;
                    tt=t+sqrt(2);
                    temp=flowacc(k+1+1,m+1+1);
                end
            end
            
            if m<ncol-1
                if flowdir(k+1,m+1+1)==O_DIR && flowacc(k+1,m+1+1)>temp
                    tk=k+0;
                    tm=m+1;
                    tt=t+1;
                    temp=flowacc(k+1,m+1+1);
                end
            end
            
            if k>0 && m<ncol-1
                if flowdir(k-1+1,m+1+1)==SO_DIR && flowacc(k-1+1,m+1+1)>temp
                    tk=k-1;
                    tm=m+1;
                    tt=t+sqrt(2);
                    temp=flowacc(k-1+1,m+1+1);
                end
                
            end
            
            if k>0
                if flowdir(k-1+1,m+1)==S_DIR && flowacc(k-1+1,m+1)>temp
                    tk=k-1;
                    tm=m+0;
                    tt=t+1;
                    temp=flowacc(k-1+1,m+1);
                end
            end
            
            if k>0 && m>0
                if flowdir(k-1+1,m-1+1)==SE_DIR && flowacc(k-1+1,m-1+1)>temp
                    tk=k-1;
                    tm=m-1;
                    tt=t+sqrt(2);
%                     temp=FLA(k-1+1,m-1+1);
                end
            end
            
            m=tm;
            k=tk;
            t=tt;
            temp=0;
            
            end %closing while loop
                
        end %closing if > threshold loop
                
        LU(i+1,j+1)=t;
        
    end %closing 2nd for loop
end %closing 1st for loop
clear i j 

% Calculation of n of junctions
z=0;
for i=0:nrow-1
    for j=0:ncol-1
        k=0;
        if LU(i+1,j+1)>1   %only channelized pixels
            
            if j>0
                if flowdir(i+1,j-1+1)==E_DIR && LU(i+1,j-1+1)>1
                    k=k+1;
                end
             end
            
            if i<nrow-1 && j>0
                if flowdir(i+1+1,j-1+1)==NE_DIR && LU(i+1+1,j-1+1)>1
                    k=k+1;
                end
            end
            
            if i<nrow-1
                if flowdir(i+1+1,j+1)==N_DIR && LU(i+1+1,j+1)>1
                    k=k+1;
                end
            end
            
            if i<nrow-1 && j<ncol-1
                if flowdir(i+1+1,j+1+1)==NO_DIR && LU(i+1+1,j+1+1)>1
                    k=k+1;
                end
            end
            
            if j<ncol-1
                if flowdir(i+1,j+1+1)==O_DIR && LU(i+1,j+1+1)>1
                    k=k+1;
                end
            end
            
            if i>0 && j<ncol-1
                if flowdir(i-1+1,j+1+1)==SO_DIR && LU(i-1+1,j+1+1)>1
                    k=k+1;
                end
            end
            
            if i>0
                if flowdir(i-1+1,j+1)==S_DIR && LU(i-1+1,j+1)>1
                    k=k+1;
                end
            end
            
            if i>0 && j>0
                if flowdir(i-1+1,j-1+1)==SE_DIR && LU(i-1+1,j-1+1)>1
                    k=k+1;
                end
            end
            
        end  %closing if channelized loop
        
        if k>1   % counter of joints
            z=z+1;   % n of junctions
        end 
    end  %closing 2nd for loop
end  %closing 1st for loop
% display(['N of junctions is: ', num2str(z)])
clear i j 
Njunction=z;


% Calculation of min lengths upstream of junctions matrix (SLJM) and vector
% (SLJ)
m=0;
% x=zeros(MAX_DRAINING_PIXEL_IN_JUNCTION,1);
SLJ=zeros(z+1,1);
% SLJ=zeros(MAX_DRAINING_PIXEL_IN_JUNCTION,1);
SLJM=zeros(nrow,ncol);  %junction min upstream length matrix:...
% 0 where NO junction, otherwise SLJ

for i=0:nrow-1
    for j=0:ncol-1
        k=0;
        x=zeros(MAX_DRAINING_PIXEL_IN_JUNCTION,1);
%         x(k+1)=0.0;
        minx=0.0;
        
        if LU(i+1,j+1)>1   %only channelized pixels
            
            if j>0
                if flowdir(i+1,j-1+1)==E_DIR && LU(i+1,j-1+1)>1
                    x(k+1)=LU(i+1,j-1+1);
                    k=k+1;
                end
            end
            if i<nrow-1 && j>0
                if flowdir(i+1+1,j-1+1)==NE_DIR && LU(i+1+1,j-1+1)>1
                    x(k+1)=LU(i+1+1,j-1+1);
                    k=k+1;
                end
            end
            
            if i<nrow-1
                if flowdir(i+1+1,j+1)==N_DIR && LU(i+1+1,j+1)>1
                    x(k+1)=LU(i+1+1,j+1);
                    k=k+1;
                end
            end
            
            if i<nrow-1 && j<ncol-1
                if flowdir(i+1+1,j+1+1)==NO_DIR && LU(i+1+1,j+1+1)>1
                    x(k+1)=LU(i+1+1,j+1+1);
                    k=k+1;
                end
            end
            
            if j<ncol-1
                if flowdir(i+1,j+1+1)==O_DIR && LU(i+1,j+1+1)>1
                    x(k+1)=LU(i+1,j+1+1);
                    k=k+1;
                end
            end
            
            if i>0 && j<ncol-1
                if flowdir(i-1+1,j+1+1)==SO_DIR && LU(i-1+1,j+1+1)>1
                    x(k+1)=LU(i-1+1,j+1+1);
                    k=k+1;
                end
            end
            
            if i>0
                if flowdir(i-1+1,j+1)==S_DIR && LU(i-1+1,j+1)>1
                    x(k+1)=LU(i-1+1,j+1);
                    k=k+1;
                end
            end
            
            if i>0 && j>0
                if flowdir(i-1+1,j-1+1)==SE_DIR && LU(i-1+1,j-1+1)>1
                    x(k+1)=LU(i-1+1,j-1+1);
                    k=k+1;
                end
            end
            
        end  %closing if channelized loop
        
        if k>1   % counter of joints
% %             minx=x(0+1);   % n of junctions
% %             for ix=1:k-1
% %                 ix=ix+1;
% %                 if minx>x(ix)
% %                     minx=x(ix);
% %                 end
% %             end
                        
            x=sort(x,'descend');
            minx=x(2);
            
            SLJ(m+1)=minx;  %after identification of a junction the min ...
%             upstream length value is stored
            m=m+1;
            SLJM(i+1,j+1)=minx;
        end 
    end  %closing 2nd for loop
end  %closing 1st for loop
clear i j 

% Calculation of max upstream length
[LUmax,~]=max(LU(:));
% % LUmax=-100;
% % for i=0:nrows-1
% %     for j=0:ncols-1
% %         if LU(i+1,j+1)>LUmax
% %             LUmax=LU(i+1,j+1);
% %         end
% %     end
% % end
% clear i j 
% display(['Max value of upstream lenght: ', num2str(LUmax)])


SLJ(m+1)=LUmax; %vector containing the min length upstream each junction and the max length of the network
m=m+1;

z=floor(LUmax)+1; %integer part of LUmax +1 to compensate truncation

SFM=zeros(z,1);  %defining modified source function (comparing the each element in the vector SLJ with numebrs from 0 to z
for i=0:z-1
    t=0;
    for j=0:m-1
        if SLJ(j+1)>i
            t=t+1;
        end
    end
    SFM(i+1)=t;
end

SFC=zeros(z,1);  %defining source function cummulative 
for i=0:z-2
    SFC(z-2-i+1)=SFC(z-1-i+1)+SFM(z-2-i+1);
end

for i=0:z-1
    SFC(i+1)=log10(SFC(i+1)*pixdim/1000);
    SFM(i+1)=log10(SFM(i+1));
end

SFM=SFM(SFM>0,:);
[s,~]=size(SFM);
SFC=SFC(1:s,:);
clear s

status=interpolation(SFC,SFM);
% display(['Parameter a_network = ', num2str(status(3))])

%%
% for each raster pixel elaboration
% SFM and SFC for each raster pixel
SLJP = zeros(m,1);
SLJPC = zeros(nrow,ncol);  %number of upstream junctions matrix
ALPHA_G = zeros(nrow,ncol).*flowdir;  %alpha matrix

CHI2P = zeros(nrow,ncol);  %chi2 matrix
DD=zeros(nrow,ncol);  %drainage density matrix
junction_to_miss = 5; %5 standard


% PRIMA DEL CALCOLO DI ALPHA PER PIXEL
for i=0:nrow-1
    for j=0:ncol-1

        if LU(i+1,j+1)>1
            SLJP=zeros(m,1);  %defining modified source function per raster pixel
            nSLJP=0;
            nFlowPath=1;
            [nSLJP,nFlowPath,SLJP]=count_junctions(flowdir,LU,SLJM,SLJP,i,j,nrow,ncol,nSLJP,nFlowPath);            
            SLJPC(i+1,j+1)=nSLJP;
            SLJP(nSLJP+1)=LU(i+1,j+1);
            nSLJP=nSLJP+1;
            DD(i+1,j+1)=nFlowPath/(flowacc(i+1,j+1)*pixdim/1000.0);
            
            if SLJPC(i+1,j+1)>junction_to_miss
%                 z=floor(LUmax)+1; %integer part of LUmax +1 to compensate truncation
                z=floor(LU(i+1,j+1))+1; %integer part of LUmax +1 to compensate truncation
                
                SFMP=zeros(z,1);
                
                for lupos=0:z-1
                    t=0;
                    for jpos=0:nSLJP-1
                        
                        if SLJP(jpos+1)>lupos
                            t=t+1;
                        end
                    end
                    
                    SFMP(lupos+1)=t;
                    
                end
                
                SFCP=zeros(z,1);
                
                for lupos=0:z-2
                    SFCP(z-2-lupos+1)=SFCP(z-1-lupos+1)+SFMP(z-2-lupos+1);
                end
                
                %alpha calculation
                for lupos=0:z-1
                    SFCP(lupos+1)=log10(SFCP(lupos+1)*pixdim/1000.0);
                    SFMP(lupos+1)=log10(SFMP(lupos+1));
                end
                
                SFMP=SFMP(SFMP>0,:);
                [s,~]=size(SFMP);
                SFCP=SFCP(1:s,:);
                clear s


                status=interpolation(SFCP,SFMP);
                ALPHA_G(i+1,j+1)=status(3);          
               
            end
        end
    end
end
clear i

i= find(ALPHA_G>0);
rete=ALPHA_G;
rete(i)= ones(size(i));


% end time
te = cputime;
fprintf('a_geomorf  : %10.3f s \n', te-ti0);

% end of function
return




