%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CONNECTIVITY METRIX
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Clocal = local connectivity (attributo locale del nodo)
% localraster = raster della local connectivity 
% Cpath = path connectivity (attributo del percorso) = prod(1-frag)
% Cnode = node connectivity (attributo del nodo rispetto tutti gli altri nodi)
% noderaster = raster della node connectivity
% Cntw = network connectivity (attributo della rete)
%
function [Clocal, Cpath, Cnode, Cntw] = ...
    connectivity (n, node_data, cnt,  distance, p, dist, metrix)

% initialize the array
Clocal = zeros(n,1);

for jn = 1:n
% calculate the local connectivity of each node
    Clocal(jn,1) = node_data(jn).cnt;
    
end

% initialize the array
Cpath = ones(n);
Cpath(logical(eye(size(Cpath)))) = 0;

switch metrix
case 1
    
% initialize the auxiliar array
aux_cnt = cnt;

% loop over the dimensions of the array
for i = 1:n
    for j = 1:n
        
        if i~=j
            path = p{i,j};
            
% loop over the number of nodes of each path
            for jp = 1:length(path)-1
                
% % if the length of the link is too shorter than the length of the overall
% % path (Llink/Lpath<0.2) then reduce the penalty of fragmentation (frag==1)
%                 if frag(path(jp),path(jp+1))==1 && (distance(path(jp),path(jp+1))/dist(i,j))<0.2
%                     aux_frag(path(jp),path(jp+1))=0;
%                 end

% calculate the path connectivity as the product of the connectivity of
% each link composing the path
                Cpath(i,j) = Cpath(i,j)*(aux_cnt(path(jp),path(jp+1)));
                                
            end
            
        end
    end
end

case 2
% initialize the array
Cpath = zeros(n,n);
for i = 1:n
    for j = 1:n
        
        if i~=j
            path = p{i,j};
            
% loop over the number of nodes of each path 
% calculate the path connectivity as mean local connectivity in the path 
        for jp = 1:length(path)-1
            Cpath(i,j) = Cpath(i,j) + cnt(path(jp),path(jp+1));
        end
        
        Cpath(i,j) = Cpath(i,j)/length(path);
        end
        
    end
end

otherwise
error ('Wrong Metrix in connectivity');
end

% Node connectivity: connectivity of each node in respect all other nodes 
Cnode = sum(Cpath,2)./(n-1);

% Network connectivity: connectivity of the network
Cntw = sum(Cnode)/n;

% end of the function
return