%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CREATE A COLORMAP WITH NINT STEPS AND N COLORS
% Inputs:
% nint = number of discretizations of each interval
% ncolor = number of colors forming the colormap
% RGB = matrix of colors [ncolor * 3]
% backRGB = vector with background color
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [mappacolori] = create_my_colormap_ncolors(nint, ncolor, RGB, backRGB)

% initialize the color map array
mappacolori = zeros(nint*(ncolor-1),3);

% linear interpolation from rgb1 to rgb2
for jcolor = 1:ncolor-1
    for i = 1:nint
        mappacolori((jcolor-1)*nint+i,:) = ...
            RGB(jcolor,:) + (i-1)/(nint-1) * (RGB(jcolor+1,:)-RGB(jcolor,:));
    end
end

% insert first color
mappacolori = [RGB(1,:); mappacolori];

% set the background if assigned
if nargin==4
    mappacolori = [backRGB; mappacolori];
end

% end of the function
return