%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CHOSE ALPHA PATTERN
% 
% Author: Silvia Garbin
% Creation: february 2017
% Last update: march 2017
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% find alpha pattern 
% (3 input) UNIFORM pattern
% (7 input) LINEAR pattern along X or Y
% (8 input) LINEAR pattern along X and Y

% find type pattern (flagTYPE)
% (1) Strong increase
% (2) Strong decrease
% (3) Weak increase
% (4) Weak decrease

% x-direction goes from flagTYPE=1 to flagTYPE=4
% y-direction goes from flagTYPE=101 to flagTYPE=104

function [alpha] = ...
    find_alpha (mean_alpha, nevent, t, event, ncol, nrow, flagTYPE1, flagTYPE2 ) 

% initialize the matrix
    alpha = zeros(nevent,1);
    
% FIND ALPHA PATTERN
switch nargin
    
case 3
    
% UNIFORM pattern of the intensity 
    for i = 1:nevent
    alpha(i) = mean_alpha;
    end
       
    if t==1
    fprintf('Uniform pattern \n');
    end
        
% LINEAR pattern ALONG X or Y
case 7
    
    alpha0 = 5;
    
    switch flagTYPE1
        case 1
            a = 0.006;
            if t==1
            fprintf('Strong increase along x \n');
            end
            
        case 2
            a = -0.006;
            if t==1
            fprintf('Strong decrease along x \n');
            end
            
        case 3
            a = 0.002;
            if t==1
            fprintf('Weak increase along x \n');
            end
            
        case 4
            a = -0.002;
            if t==1
            fprintf('Weak decrease along x \n');
            end
            
        case 101
            b = 0.006;
            if t==1
            fprintf('Strong increase along y \n');
            end
            
        case 102
            b = -0.006;
            if t==1
            fprintf('Strong decrease along y \n');
            end
            
        case 103
            b = 0.002;
            if t==1
            fprintf('Weak increase along y \n');
            end
            
        case 104
            b = -0.002;
            if t==1
            fprintf('Weak decrease along y \n');
            end
            
        otherwise
            error ('WrongFlagTYPE1 in find_alpha')

    end
    
    if flagTYPE1 <= 4
        if a>0 && t==1
            fprintf('Pattern: Linear W-E \n');
        elseif a<0 && t==1
            fprintf('Pattern: Linear E-W \n');
        end
        
            for i = 1:nevent
                alpha(i) = mean_alpha+a*alpha0*(event(i,2)-ncol/2);
                
% verify that alpha velue is between 5 and 30 mm
                alpha(i) = max(min(alpha(i),30),5);               
            end
    else
        if b>0 && t==1
            fprintf('Pattern: Linear N-S \n');
        elseif b<0 && t==1
            fprintf('Pattern: Linear S-N \n');
        end
        
            for i = 1:nevent
                alpha(i) = mean_alpha+b*alpha0*(event(i,1)-nrow/2);
                
% verify that alpha velue is between 5 and 30 mm
                alpha(i) = max(min(alpha(i),30),5);
            end
    end

% LINEAR pattern ALONG X and Y
case 8
    
    alpha0 = 5;
    
    switch flagTYPE1
        case 1
            a = 0.006;
            if t==1
            fprintf('Strong increase along x \n');
            end
            
        case 2
            a = -0.006;
            if t==1
            fprintf('Strong decrease along x \n');
            end
            
        case 3
            a = 0.002;
            if t==1
            fprintf('Weak increase along x \n');
            end
            
        case 4
            a = -0.002;
            if t==1
            fprintf('Weak decrease along x \n');
            end
    
        otherwise
            error ('WrongFlagTYPE1 in find_alpha')
    end
    
    switch flagTYPE2
        case 101
            b = 0.006;
            if t==1
            fprintf('Strong increase along y \n');
            end
            
        case 102
            b = -0.006;
            if t==1
            fprintf('Strong decrease along y \n');
            end
            
        case 103
            b = 0.002;
            if t==1
            fprintf('Weak increase along y \n');
            end
            
        case 104
            b = -0.002;
            if t==1
            fprintf('Weak decrease along y \n');
            end
            
        otherwise
            error ('WrongFlagTYPE2 in find_alpha')

    end

    
    if a==b 
        if a>0 && t==1
            fprintf('Pattern: Linear S-E \n');
        elseif a<0 && t==1
            fprintf('Pattern: Linear N-W \n');
        end
    else
        if a>0 && t==1
            fprintf('Pattern: Linear N-E \n');
        elseif a<0 && t==1
            fprintf('Pattern: Linear S-W \n');
        end
    end
          
    for i = 1:nevent
        alpha(i) = ...
            mean_alpha+a*alpha0*(event(i,2)-ncol/2)+b*alpha0*(event(i,1)-nrow/2)+...
            a*b*(event(i,2)*event(i,1)-ncol*nrow/4);

% verify that alpha velue is between 5 and 30 mm
        alpha(i) = max(min(alpha(i),30),5);
    end

% error in the input flag
    otherwise
        error ('Wrong number of inputs in find_alpha');
end


% end of function
return