%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIND CHANNEL HEAD
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function channel_heads = find_head (rete, nrow, ncol, scarico)

% initialize time
ti0 = cputime;

% initialize the array
head = repmat(struct('i',0,'j',0,'k',0),0, 1); 

nhead = 1;

for ihead = 1:nrow
    for jhead = 1:ncol
        if rete(ihead,jhead) == 1
            
            k_rete = sub2ind(size(rete),ihead,jhead);
            
            % list neighbours values in terms of rete
            neigh_rete = ([rete(ihead-1:ihead+1,jhead-1); rete(ihead-1,jhead);...
                rete(ihead+1,jhead); rete(ihead-1:ihead+1,jhead+1)]);
            
            % list neighbours values in terms of k linear-index
            neigh_k = ([sub2ind(size(rete),ihead-1,jhead-1); sub2ind(size(rete),ihead,jhead-1);...
                sub2ind(size(rete),ihead+1,jhead-1); sub2ind(size(rete),ihead-1,jhead); ...
                sub2ind(size(rete),ihead+1,jhead); sub2ind(size(rete),ihead-1,jhead+1);...
                sub2ind(size(rete),ihead,jhead+1); sub2ind(size(rete),ihead+1,jhead+1)]);
            
            % if none of neighbours drein into the current pixel then
            % current pixel is a channel head
            if ~ismember(k_rete,scarico(neigh_k(neigh_rete==1)))
                
                head(nhead).i = ihead;
                head(nhead).j = jhead;
                head(nhead).k = sub2ind(size(rete), ihead, jhead);
            % update the counter
            nhead = nhead + 1;
            end
            
            clear neigh
        end
    end
end

nchead = max(size(head));

channel_heads = repmat(struct('i',0,'j',0,'k',0),nchead, 1); 
for iaux = 1:nchead
        if head(iaux).i ~= 0 && ...
                head(iaux).j ~= 0 && ...
                head(iaux).k ~= 0
            
           channel_heads(iaux).i = head(iaux).i;
           channel_heads(iaux).j = head(iaux).j;
           channel_heads(iaux).k = head(iaux).k;
        end
end


% end time
te = cputime;
fprintf('heads      : %10.3f s \n', te-ti0);

% end of function
return
