
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIND THE NORMALIZATION CONSTANT OF PDF (linear and non linear)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [nc_q, check, norm_atom] = find_normconst(alphaG, alphaP, kappa, lambda)

clear x

if alphaG == 1
    
    norm_atom = 0;
    
% LINEAR STREAMFLOW PDF
    LpdfQNB_an = @(x)(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa))*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1);
    
%     fprintf ('LpdfQNB_an(0.01) = %f \nLpdfQNB_an(1000) = %f \n', LpdfQNB_an(0.01), LpdfQNB_an(1000))
    
    nc = quadgk(LpdfQNB_an,0,Inf);
    nc_q=1/nc;
    clear LpdfQan x

% Computing the analytical linear streamflow pdf
    LpdfQ_an = @(x) nc_q.*(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa))*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1);
    
% fprintf ('LpdfQ_an (0.01) = %f \nLpdfQ_an (1000) = %f \n', LpdfQ_an (0.01), LpdfQ_an (1000))

% Computing the analytical linear streamflow cdf
    LcdfQ_an = @(x) quadgk(LpdfQ_an,0.0,x);
    check = LcdfQ_an(Inf);
    
else
%--------------------------------------------------------------------------
% NON-LINEAR STREAMFLOW PDF    

% calculate the pdf
    gammaW = 1/alphaP;

% calculate the continue part of the pdf
    pdf_continua = @(x)((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
        ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG))))); 

    if alphaG < 1
% there is an atom of probability
        atom = kappa/lambda;
    else
        atom = 0;
    end

    nc = quadgk(pdf_continua,0,Inf) + atom;

% update the normalization constant
    nc_q = 1/nc;
    
% calculate the normalized atom
    norm_atom = atom*nc_q;
 

%% check the normalization constant
    clear pdf x 

    pdf_check = @(x) nc_q.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
    ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG)))));

    cdf_check = @(x) quadgk(pdf_check,0,x) + nc_q.*atom; % from Botter et al., 2009
 
% check = cdf_check;
    check = cdf_check(Inf);

% figure()
% subplot(1,2,1)
% fplot(pdf_check,[0,7])
% ylabel('PDF(q)[-]')
% xlabel('q[mm/d]')
% subplot(1,2,2)
% fplot(cdf_check,[0,5])
% ylabel('CDF(q)[-]')
% xlabel('q[mm/d]')

    clear pdf_check
end


%end of the function
return
