function [ncH, check, pdfH_an, cdf_frag, h_media, meanHS, var, kur] = pdf_H_BIS...
    (ncQ, alphaG, alphaP, kappa, lambda, h0, delta, fla, pxl_dim, hstar, jn)

clear x

if alphaG == 1
    
% LINEAR STAGE PDF
    LpdfHNB_an = @(x) ncQ.*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6))./(alphaP.*kappa)).*...
        ((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(lambda./kappa-1);
    nc = quadgk(LpdfHNB_an,0,Inf);
    ncH=1/nc;
    clear LpdfHNB_an x

% Computing the analytical linear stage pdf
    pdfH_an = @(x) ncH.*ncQ.*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6))./(alphaP.*kappa)).*...
        ((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(lambda./kappa-1);

% Computing the analytical linear stage cdf
    cdfH_an = @(x) quadgk(pdfH_an,0,x);
    check = cdfH_an(Inf);
    
% Computing the analytical linear fragmentation index
    cdf = @(x) quadgk(pdfH_an,0,Inf) - quadgk(pdfH_an,x,Inf);
    
    cdf_frag = cdf(hstar);
    
% Computing the mean of the distribution
    media = @(x) ncH.*ncQ.*x.*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6))./(alphaP.*kappa)).*...
        ((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(lambda./kappa-1);

    h_media = quadgk(media,0,Inf);
    
% Computing the variance of the distribution
    variance = @(x) ncH.*ncQ.*((x-h_media).^2).*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6))./(alphaP.*kappa)).*...
        ((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(lambda./kappa-1);

    var = quadgk(variance,0,Inf);
    
    
% Computing the kurtosis of the distribution
    kurtosis = @(x) ncH.*ncQ.*((x-h_media).^4).*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6))./(alphaP.*kappa)).*...
        ((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(lambda./kappa-1);

    kur = quadgk(kurtosis,0,Inf);
    
else

%--------------------------------------------------------------------------
% NON-LINEAR STREAMFLOW PDF    

    gammaW = 1/alphaP;

    if alphaG < 1
% there is an atom of probability
        atom = kappa/lambda;
    else
        atom = 0;
    end

    NLpdfHNB_an = @(x) ncQ.*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        ((1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
        exp(((-gammaW/(kappa*(2-alphaG))).*(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG))) +...
        ((lambda/(kappa*(1-alphaG))).*(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG)))));
    nc = quadgk(NLpdfHNB_an,0,Inf) + ncQ*atom;
    ncH = 1/nc;
    clear NLpdfHNB_an x
    
% Computing the analytical linear stage pdf
    pdfH_an = @(x) ncH.*ncQ.*((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        ((1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
        exp(((-gammaW/(kappa*(2-alphaG))).*(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG))) +...
        ((lambda/(kappa*(1-alphaG))).*(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG)))));

% Computing the analytical linear stage cdf
    cdfH_an = @(x) quadgk(pdfH_an,0,x) + ncH*ncQ*atom;
    check = cdfH_an(Inf);
    
% Computing the analytical linear fragmentation index
    cdf = @(x) quadgk(pdfH_an,0,Inf) - quadgk(pdfH_an,x,Inf) + ncH*ncQ*atom; 
    
    cdf_frag = cdf(hstar);  
    
% Computing the mean of the distribution
%     media = @(x) ncH.*ncQ.*x.*(((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
%         (1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
%         exp((-gammaW/(kappa*(2-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG) +...
%         (lambda/(kappa*(1-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG))+atom);
        
    media = @(x) ncH.*ncQ.*x.*(((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
        exp((-gammaW/(kappa*(2-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG) +...
        (lambda/(kappa*(1-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG)));
    
    h_media = quadgk(media,0,Inf); 
    
    
% Computing the variance of the distribution    
    variance = @(x) ncH.*ncQ.*((x-h_media).^2).*(((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
        exp((-gammaW/(kappa*(2-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG) +...
        (lambda/(kappa*(1-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG)));
    
    var = quadgk(variance,0,Inf); 
    
    
% Computing the kurtosis of the distribution    
    kurtosis = @(x) ncH.*ncQ.*((x-h_media).^4).*(((x./h0).^(1./delta)./(delta.*x.*fla.*pxl_dim^2.*10^6)).*...
        (1./(((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^alphaG)).*...
        exp((-gammaW/(kappa*(2-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(2-alphaG) +...
        (lambda/(kappa*(1-alphaG))).*((x./h0).^(1./delta)./(fla.*pxl_dim^2.*10^6)).^(1-alphaG)));
    
    kur = quadgk(kurtosis,0,Inf); 
    
end


%%
if alphaG == 1
    
    atom = 0;
    
    pdfQ = @(x) ncQ.*(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1);
    
else

    if alphaG < 1
    % there is an atom of probability
        atom = kappa/lambda;
    else
        atom = 0;
    end
    pdfQ = @(x) ncQ.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
        ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG))))+atom);
end

% set mean habitat suitability: HS(q)=C*exp(-Bx)x^(A-1)
% set function parameters
A = 1.042;
B = 5.62;
C = 1.28;

if alphaG == 1

    % Computing the mean habitat suitability
    fun = @(x) (ncQ.*(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa))*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1)).*C.*exp(-B.*x).*x.^(A-1);
    meanHS = quadgk(fun,0,Inf);
else
    if alphaG < 1
% there is an atom of probability
        atom = kappa/lambda;
    else
        atom = 0;
    end
          
% Computing the mean habitat suitability
    fun = @(x) ncQ.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
    ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG))))+atom).*C.*exp(-B.*x).*x.^(A-1);
    meanHS = quadgk(fun,0,Inf);   
    
end
    



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT PDF(h) and CDF(h)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
% set(figure1,'WindowStyle','docked')
% subplot (1,2,1)
% fplot(pdfH_an,[0 500]);
% hold on
% if atom>0
% plot(0,ncQ*atom,'*');
% end
% subplot (1,2,2)
% fplot(cdfH_an,[0 500]);
% hold on
% if atom>0
% plot(0,ncH*ncQ*atom,'*');
% end


% figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
% set(figure1,'WindowStyle','docked')
% fplot(cdf_check,[0 100]);

%%
% pdfQ = @(x) ncQ.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
%     ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG)))));
% atomo = ncQ*atom;

% figure()
% set(figure1,'WindowStyle','docked')
% subplot(1,2,1)
% fplot(pdfQ,[0 50]);
% hold on
% if atomo>0
% plot(0,atomo,'*');
% end
% subplot(1,2,2)
% fplot(pdf_check,[0 50]);


%% 
% figure()
% set(figure1,'WindowStyle','docked')
% subplot(1,2,1)
% fplot(pdf_check,[0,700])
% ylabel('PDF(q)[-]')
% xlabel('q[mm/d]')
% % subplot(1,2,2)
% fplot(cdf_check,[0,5])
% ylabel('CDF(q)[-]')
% xlabel('q[mm/d]')


%%
% PLOT PDF(h) and CDF(h)
% 
% figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
% set(figure1,'WindowStyle','docked')
% subplot (1,2,1)
% fplot(pdf_check,[0 300])
% ylabel('PDF(hstar)[1/mm]')
% xlabel('hstar[mm]')
% subplot (1,2,2)
% fplot(cdf_check,[0 300])
% ylabel('CDF(hstar)[-]')
% xlabel('hstar[mm]')

%%
%  plot the pdf function of the outlet
% if jn == 26
% %     figure('units','normalized','outerposition',[0.3 0.28 0.94 0.69]);
% %     figure2 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
%     
%     fplot (pdf_check,[0,50])
%     title ('<P> = 500 mm/y and \lambda_{P} = 1 d^{-1}')
%     ylabel('PDF(hstar)[1/mm]')
%     xlabel('hstar[mm]')
%     
%     
% end
%     saveas(gcf,'pdf(h)1.png')
%     figure3 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
%     fplot (cdf,[0,700])
%     title ('<P> = 500 mm/y and \lambda_{P} = 1 d^{-1}')
%     ylabel('frag(hstar)[-]')
%     xlabel('hstar[mm]')
%     saveas(gcf,'cdf(h)1.png')
% 
% end


%     set(gca,'yscale','log')
%     grid on

%%

% end of the function
return