%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% LINEAR & NON lINEAR PDF and CDF
% This function calculates:
% 1) the pdf(q) and the cdf(q);
% 2) the mean of the pdf(q).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [an_pdf, cdf, q_media] = pdf_Q(nc_q, alphaG, alphaP, kappa, lambda, q, jn)

clear x

if alphaG == 1
    
    atom = 0;
    
% Computing the analytical linear streamflow pdf
    an_pdf = @(x) nc_q.*(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa))*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1);

% Computing the analytical linear streamflow cdf
    an_cdf = @(x) quadgk(an_pdf,0,Inf) - quadgk(an_pdf,x,Inf);
    cdf = an_cdf(q);
    
% Computing the mean of the distribution
    media = @(x) nc_q.*x.*(((alphaP.*kappa).^(-lambda./kappa))./gamma(lambda./kappa)).*...
        exp(-x./(alphaP.*kappa)).*x.^(lambda./kappa-1);

    q_media = quadgk(media,0,Inf); 
    
else
    
%--------------------------------------------------------------------------
% NON-LINEAR STREAMFLOW PDF    
        
    gammaW = 1/alphaP; 
    
% Computing the analytical non linear streamflow pdf
    an_pdf = @(x) nc_q.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
    ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG)))));

    if alphaG < 1
% there is an atom of probability
        atom = kappa/lambda;
    else
        atom = 0;
    end

% Computing the analytical non linear streamflow cdf
    an_cdf = @(x) quadgk(an_pdf,0,Inf) - quadgk(an_pdf,x,Inf) + nc_q.*atom;
%     cdf = an_cdf(q);
    cdf = an_cdf(100);
    
    
% Computing the mean of the distribution
    media = @(x) nc_q.*x.*((1./(x.^alphaG)).* exp(((-gammaW/(kappa*(2-alphaG))).*(x.^(2-alphaG))) +...
        ((lambda/(kappa*(1-alphaG))).*(x.^(1-alphaG))))+atom);

    q_media = quadgk(media,0,Inf); 
   

end

%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% PLOT PDF(q) and CDF(q)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
% set(figure1,'WindowStyle','docked')
% fplot(an_pdf,[0 2]);
% hold on
% if atom>0
% plot(0,nc_q.*atom,'*');
% end


% figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
% set(figure1,'WindowStyle','docked')
% fplot(cdf,[0 1]);

%%
% if jn==1
% figure()
% subplot(1,2,1)
% fplot(pdf_continue,[0,0.0001])
% ylabel('PDF(q)[-]')
% end
% xlabel('q[mm/d]')
% subplot(1,2,2)
% fplot(cdf,[0,1000])
% ylabel('CDF(q)[-]')
% xlabel('q[mm/d]')

% % plot the pdf function of the outlet
% if jn==17
%     figure1 = figure('units','normalized','outerposition',[0.01 0.01 0.98 0.98]);
%     fplot (pdf,[0,3])
%     title ('<P> = 500 mm/y and \lambda_{P} = 1 d^{-1}')
%     ylabel('PDF(q)[d/mm]')
%     xlabel('q[mm/d]')
%     saveas(gcf,'pdf(q)1.png')
% end

% set(gca,'xscale','log')
% grid on

% end of the function
return