%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% FIND THE UPSTREAM AREA FOR EACH NODE
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [matrix_subbasin_nodes] = ...
    subbasins_nodes(nrow, ncol, npath, percorsi, noderaster, nnode)
% initialize time
ti0 = cputime;

% initialize the array
matrix_subbasin_nodes = repmat(struct('i',0,'j',0,'k',0,'m',0,'fla',0), nnode, 1);
jpch = 0;

% build the pointer vector to find the positions into the matrix
% containing the upstream subbasin of the channel pixel, i.e. 
% matrix_subbasin_channel
jpixel2node = nan(nrow*ncol,1);

% initialize time
ti = cputime;

% loop over the paths
for jpath = 1:npath
    
% consider the current path (i.e. from a source pixel to the outlet), and
% build the upstream draining matrix for each node of the path

% initialize auxiliary matrix
    aux = nan(nrow,ncol);

% find the length of the current path
    npxl = length(percorsi(jpath).p);
    
% loop over the pixel of the current path
    for jpxl = 1:npxl
        
% find the current pixel
        knode = percorsi(jpath).p(jpxl);
        
% find the indices of the matrix
        [i,j] = ind2sub(size(noderaster),knode);
        
% save the current in the auxiliary matrix
        aux(i,j) = 1;
        
% if we are on a node, then save the upstream basin
        if noderaster(i,j)== -1 

% this pixel is new, thus create the matrix
            if isnan(jpixel2node(knode))
                
% update the pointer
                jpch = jpch + 1;

% save the current upstream basin
                jpixel2node(knode) = jpch;
                matrix_subbasin_nodes(jpch).i = i;
                matrix_subbasin_nodes(jpch).j = j;
                matrix_subbasin_nodes(jpch).k = knode;
                matrix_subbasin_nodes(jpch).m = aux;
                
% this channel node already exists, thus update the matrix
            else
                jaux = jpixel2node(knode);
                matrix_subbasin_nodes(jaux).m(isnan(matrix_subbasin_nodes(jaux).m)) = ...
                    aux(isnan(matrix_subbasin_nodes(jaux).m));
            end
        end
                
% end of the loops over the entire basin
    end
    
% print status
    jprint = 100;
    if mod(jpath,jprint)==0
        te = cputime;
        fprintf('  path %u of %u (%6.2f %%): %8.2f s (%8.4f s) \n', ...
            jpath, npath, jpath/npath*100,te-ti,(te-ti)/jprint);
%         if jpath==20
%             return
%         end
        ti = cputime;
    elseif  jpath == npath
        te = cputime;
        fprintf('  path %u of %u (%6.2f %%): %8.2f s (%8.4f s) \n', ...
            jpath, npath, jpath/npath*100,te-ti,(te-ti)/mod(jpath,jprint));
    end
end

% save backup file
if (0)
	save('TMP_matrix_subbasin_nodes','matrix_subbasin_nodes');
end

%% SET THE CURRENT PIXEL AS ZERO
for jn = 1:nnode
    jrow = matrix_subbasin_nodes(jn).i;
    jcol = matrix_subbasin_nodes(jn).j;
    matrix_subbasin_nodes(jn).m(jrow,jcol) = 0;
    matrix_subbasin_nodes(jn).fla = nansum(nansum(matrix_subbasin_nodes(jn).m))+1;
end

%% SORT WITH RESPECT TO INDEX K
for jch1 = 1:nnode-1
    for jch2 = jch1+1:nnode
        if matrix_subbasin_nodes(jch2).k < matrix_subbasin_nodes(jch1).k
%             aux_i = matrix_subbasin_nodes(jch2).i;
%             aux_j = matrix_subbasin_nodes(jch2).j;
%             aux_k = matrix_subbasin_nodes(jch2).k;
%             aux_m = matrix_subbasin_nodes(jch2).m;
%             aux_fla = matrix_subbasin_nodes(jch2).fla;
%
%             matrix_subbasin_nodes(jch2).i = matrix_subbasin_nodes(jch1).i;
%             matrix_subbasin_nodes(jch2).j = matrix_subbasin_nodes(jch1).j;
%             matrix_subbasin_nodes(jch2).k = matrix_subbasin_nodes(jch1).k;
%             matrix_subbasin_nodes(jch2).m = matrix_subbasin_nodes(jch1).m;
%             matrix_subbasin_nodes(jch2).fla = matrix_subbasin_nodes(jch1).fla;
% 
%             matrix_subbasin_nodes(jch1).i = aux_i;
%             matrix_subbasin_nodes(jch1).j = aux_j;
%             matrix_subbasin_nodes(jch1).k = aux_k;
%             matrix_subbasin_nodes(jch1).m = aux_m;
%             matrix_subbasin_nodes(jch1).fla = aux_fla;

            aux = matrix_subbasin_nodes(jch2);
            matrix_subbasin_nodes(jch2) = matrix_subbasin_nodes(jch1);
            matrix_subbasin_nodes(jch1) = aux;
        end
    end
end



% end time
te = cputime;
fprintf('subbasins  : %10.3f s \n', te-ti0);

% end of the function
return